/***************************************************************************
 *   Copyright (C) 2008  Antony Martin <martin(dot)antony(at)yahoo(dot)fr> *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 19 janvier 2007
   derniere modification le 22/11/2008 */

#ifndef SCESCENE_H
#define SCESCENE_H

#include <SCE/utils/SCEList.h>
/*#include <SCE/interface/SCEState.h>*/
#include <SCE/interface/SCENode.h>
#include <SCE/interface/SCECamera.h>
#include <SCE/interface/SCETexture.h>
#include <SCE/interface/SCEMaterial.h>
/*#include <SCE/interface/SCEShaders.h>*/
#include <SCE/interface/SCESceneEntity.h>

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * \ingroup scene
 * @{
 */

#define SCE_SCENE_SHADERS_GROUP 0
#define SCE_SCENE_MATERIALS_GROUP 1
#define SCE_SCENE_TEXTURES0_GROUP 2
#define SCE_SCENE_NUM_RESOURCE_GROUP 3

typedef int (*SCE_FSceneForEachEntityGroupFunc)(SCE_SSceneEntityGroup *g,
                                                void *p);

/** \copydoc sce_sscenestates */
typedef struct sce_sscenestates SCE_SSceneStates;
/**
 * \brief States of a scene
 */
struct sce_sscenestates
{
    int clearcolor, cleardepth; /**< Do we clear the buffers ? */
    int frustum_culling; /**< Use or not the frustum culling */
    int use_skybox;      /**< Use or not a skybox */
    int lighting;        /**< Use or not lighting */
    int lod;             /**< Use or not LOD for scene's models */
};

/* scene */
/** \copydoc sce_sscene */
typedef struct sce_sscene SCE_SScene;
/**
 * \brief Scene description structure
 */
struct sce_sscene
{
    SCE_SNode *rootnode;        /**< Root node */
    int node_updated;           /**< Defines the root node's status */

    /** All the resources of the scene */
    SCE_SSceneResourceGroup *rgroups[SCE_SCENE_NUM_RESOURCE_GROUP];

    SCE_SList *egroups;         /**< Scene's entities groups list */
    SCE_SList *entities;        /**< Scene's entities */
    SCE_SList *lights;          /**< Scene's lights list */

    /** Color buffer clear values */
    float rclear, gclear, bclear, aclear;
    float dclear;               /**< Depth buffer clear value */
    SCE_STexture *rendertarget; /**< Scene's render target */
    SCEuint cubeface;           /**< Face of the cubemap (used if
                                 * sce_sscenerenderinfos::rendertarget is
                                 * a cubemap) */
    SCE_SCamera *camera;        /**< The camera for the current render */
    SCE_SSceneStates states;    /**< Scene's state */
};

/** @} */

/* initialise le gestionnaire de scenes */
int SCE_Init_Scene (void);
/* quitte le gestionnaire de scenes */
void SCE_Quit_Scene (void);

/* cree une nouvelle scene */
SCE_SScene* SCE_Scene_Create (void);
/* supprime une scene */
void SCE_Scene_Delete (SCE_SScene*);

/* retourne le noeud racine */
SCE_SNode* SCE_Scene_GetRootNode (SCE_SScene*);

/* ajoute un noeud au noeud racine d'une scene */
void SCE_Scene_AddNode (SCE_SScene*, SCE_SNode*);
/* ajoute une entite a une scene */
int SCE_Scene_AddEntity (SCE_SScene*, SCE_SSceneEntity*);
/* ajoute un groupe d'entites a une scene */
int SCE_Scene_AddEntityGroup (SCE_SScene*, SCE_SSceneEntityGroup*);
/* ajoute une lumiere a une scene */
int SCE_Scene_AddLight (SCE_SScene*, SCE_SLight*);
/* ajoute une ressource a une scene */
void SCE_Scene_AddResource (SCE_SScene*, int, SCE_SSceneResource*);

/* execute une fonction pour chaque entite de la scene */
void SCE_Scene_ForEachEntity (SCE_SScene*, SCE_FSceneForEachEntityGroupFunc,
                              void*);

int SCE_Scene_SetupBatching (SCE_SScene*, unsigned int, int*);
int SCE_Scene_SetupDefaultBatching (SCE_SScene*);

/* defini le commencement ou la fin d'une frame */
void SCE_Scene_BeginFrame (SCE_SScene*);
void SCE_Scene_EndFrame (SCE_SScene*);

/* vide les tampons en preparation d'un rendu */
void SCE_Scene_ClearBuffers (SCE_SScene*);

/* met a jour la scene et stocke les informations de rendu dans
   l'element courant de la liste des infos de rendu */
void SCE_Scene_Update (SCE_SScene*, SCE_SCamera*, SCE_STexture*, SCEuint);
/* rend une scene a partir des infos courantes de rendu */
void SCE_Scene_Render (SCE_SScene*, SCE_SCamera*, SCE_STexture*, int);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* guard */
