/***************************************************************************
 *   Copyright (C) 2008  Antony Martin <martin(dot)antony(at)yahoo(dot)fr> *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 26/02/2008
   derniere modification : 27/02/2008 */

#include <SCE/utils/SCERectangle.h>

/**
 * \file SCERectangle.c
 * \copydoc rectangle
 * \file SCERectangle.h
 * \copydoc rectangle
 */

/**
 * \defgroup rectangle Rectangles managment
 * \ingroup utils
 * \brief Rectangle managment functions
 */

/** @{ */

/* ajoute le 26/02/2008 */
/**
 * \brief Initialize a SCE_SIntRect
 * \param r a SCE_SIntRect to initialize
 */
void SCE_Rectangle_Init (SCE_SIntRect *r)
{
    r->x1 = r->y1 = r->x2 = r->y2 = 0;
}
/**
 * \brief Initialize a SCE_SFloatRect
 * \param r a SCE_SFloatRect to initialize
 */
void SCE_Rectangle_Initf (SCE_SFloatRect *r)
{
    r->x1 = r->y1 = r->x2 = r->y2 = 0.0f;
}

/* ajoute le 27/02/2008 */
/**
 * \brief Defines a rectange
 * \param r the rectangle to define
 * \param x1 X coordinate of a corner
 * \param y1 Y coordinate of a corner
 * \param x2 X coordinate of the opposite corner
 * \param y2 Y coordinate of the opposite corner
 * \see SCE_Rectangle_Setf()
 */
void SCE_Rectangle_Set (SCE_SIntRect *r, int x1, int y1, int x2, int y2)
{
    r->x1 = x1; r->x2 = x2; r->y1 = y1; r->y2 = y2;
}
/**
 * \brief Defines a rectange
 * \param r the rectangle to define
 * \param x1 X coordinate of the bottom left corner
 * \param y1 Y coordinate of the bottom left corner
 * \param x2 X coordinate of the top right corner
 * \param y2 Y coordinate of the top right corner
 * \see SCE_Rectangle_Set()
 * \warning The firsts point cordinates must be lesser than the second point
 * coordinates
 */
void SCE_Rectangle_Setf (SCE_SFloatRect *r, float x1, float y1,
                         float x2, float y2)
{
    r->x1 = x1; r->x2 = x2; r->y1 = y1; r->y2 = y2;
}

/* ajoute le 27/02/2008 */
/**
 * \brief Moves a rectange
 * \param r the rectangle to move
 * \param x X offset
 * \param y Y offset
 * \see SCE_Rectangle_Movev()
 */
void SCE_Rectangle_Move (SCE_SIntRect *r, int x, int y)
{
    r->x1 += x; r->x2 += x;
    r->y1 += y; r->y2 += y;
}
/**
 * \brief Moves a rectange
 * \param r the rectangle to move
 * \param x X offset
 * \param y Y offset
 * \see SCE_Rectangle_Movefv()
 */
void SCE_Rectangle_Movef (SCE_SFloatRect *r, float x, float y)
{
    r->x1 += x; r->x2 += x;
    r->y1 += y; r->y2 += y;
}
/**
 * \brief Moves a rectange
 * \param r the rectangle to move
 * \param v vector offset
 * \see SCE_Rectangle_Move()
 */
void SCE_Rectangle_Movev (SCE_SIntRect *r, int *v)
{
    SCE_Rectangle_Move (r, v[0], v[1]);
}
/**
 * \brief Moves a rectange
 * \param r the rectangle to move
 * \param v vector offset
 * \see SCE_Rectangle_Movef()
 */
void SCE_Rectangle_Movefv (SCE_SFloatRect *r, SCE_TVector2 v)
{
    SCE_Rectangle_Movef (r, v[0], v[1]);
}

/* ajoute le 26/02/2008 */
/**
 * \brief Gets the rectange area
 * \param r a rectangle
 * \return the rectangle area
 */
int SCE_Rectangle_GetArea (SCE_SIntRect *r)
{
    return (r->x2 - r->x1) * (r->y2 - r->y1);
}
/**
 * \brief Gets the rectange area
 * \param r a rectangle
 * \return the rectangle area
 * 
 * \see SCE_Rectangle_GetArea()
 */
float SCE_Rectangle_GetAreaf (SCE_SFloatRect *r)
{
    return (r->x2 - r->x1) * (r->y2 - r->y1);
}

/* ajoute le 27/02/2008 */
/**
 * \brief Gets the rectange width
 * \param r a rectangle
 * \return the rectangle width
 */
int SCE_Rectangle_GetWidth (SCE_SIntRect *r)
{
    return r->x2 - r->x1;
}
/**
 * \brief Gets the rectange width
 * \param r a rectangle
 * \return the rectangle width
 * 
 * \see SCE_Rectangle_GetWidth()
 */
float SCE_Rectangle_GetWidthf (SCE_SFloatRect *r)
{
    return r->x2 - r->x1;
}
/* ajoute le 27/02/2008 */
/**
 * \brief Gets the rectange height
 * \param r a rectangle
 * \return the rectangle height
 */
int SCE_Rectangle_GetHeight (SCE_SIntRect *r)
{
    return r->y2 - r->y1;
}
/**
 * \brief Gets the rectange height
 * \param r a rectangle
 * \return the rectangle height
 * 
 * \see SCE_Rectangle_GetHeight()
 */
float SCE_Rectangle_GetHeightf (SCE_SFloatRect *r)
{
    return r->y2 - r->y1;
}

/* ajoute le 27/02/2008 */
/**
 * \brief Compares two rectangles
 * \param a a rectangle
 * \param b another rectangle
 * \return SCE_TRUE if the two rectangles are equal, SCE_FALSE otherwise
 */
int SCE_Rectangle_Equal (SCE_SIntRect *a, SCE_SIntRect *b)
{
    return (a->x1==b->x1 && a->y1==b->y1 && a->x2==b->x2 && a->y2==b->y2);
}
/**
 * \brief Compares two rectangles
 * \param a a rectangle
 * \param b another rectangle
 * \return SCE_TRUE if the two rectangles are equal, SCE_FALSE otherwise
 * 
 * \see SCE_Rectangle_Equal()
 */
int SCE_Rectangle_Equalf (SCE_SFloatRect *a, SCE_SFloatRect *b)
{
#define EPS 0.000002 /* "epsilon" (pux) */
    return (fabs (a->x1 - b->x1) < EPS && fabs (a->y1 - b->y1) < EPS &&
            fabs (a->x2 - b->x2) < EPS && fabs (a->y2 - b->y2) < EPS);
}

/* ajoute le 26/02/2008 */
/**
 * \brief Check if a point is inside a rectangle
 * \param r a rectangle
 * \param x X coordinates of the point
 * \param y Y coordinates of the point
 * \return SCE_TRUE if the point is inside teh rectangles, SCE_FALSE otherwise
 */
int SCE_Rectangle_IsIn (SCE_SIntRect *r, int x, int y)
{
    return (x > r->x1 && x < r->x2 && y > r->y1 && x < r->y2);
}
/**
 * \brief Check if a point is inside a rectangle
 * \param r a rectangle
 * \param x X coordinates of the point
 * \param y Y coordinates of the point
 * \return SCE_TRUE if the point is inside teh rectangles, SCE_FALSE otherwise
 * 
 * \see SCE_Rectangle_IsIn()
 */
int SCE_Rectangle_IsInf (SCE_SFloatRect *r, float x, float y)
{
    return (x > r->x1 && x < r->x2 && y > r->y1 && x < r->y2);
}
/**
 * \brief Check if a point is inside a rectangle
 * \param r a rectangle
 * \param p vector coordinates of the point
 * \return SCE_TRUE if the point is inside teh rectangles, SCE_FALSE otherwise
 * 
 * \see SCE_Rectangle_IsIn()
 */
int SCE_Rectangle_IsInv (SCE_SIntRect *r, int *p)
{
    return SCE_Rectangle_IsIn (r, p[0], p[1]);
}
/**
 * \brief Check if a point is inside a rectangle
 * \param r a rectangle
 * \param p vector coordinates of the point
 * \return SCE_TRUE if the point is inside teh rectangles, SCE_FALSE otherwise
 * 
 * \see SCE_Rectangle_IsInf()
 */
int SCE_Rectangle_IsInfv (SCE_SFloatRect *r, float *p)
{
    return SCE_Rectangle_IsInf (r, p[0], p[1]);
}

/* ajoute le 27/02/2008 */
/**
 * \brief Gets the rectangle's edges
 * \param re a rectangle
 * \param l a SCE_SLine where store the left edge
 * \param r a SCE_SLine where store the right edge
 * \param b a SCE_SLine where store the bottom edge
 * \param t a SCE_SLine where store the top edge
 */
void SCE_Rectangle_MakeLines (SCE_SIntRect *re, SCE_SLine *l, SCE_SLine *r,
                              SCE_SLine *b, SCE_SLine *t)
{
    SCE_Line_Set (l, (int)re->x1, (int)re->y1, (int)re->x1, (int)re->y2);
    SCE_Line_Set (r, (int)re->x2, (int)re->y1, (int)re->x2, (int)re->y2);
    SCE_Line_Set (b, (int)re->x1, (int)re->y1, (int)re->x2, (int)re->y1);
    SCE_Line_Set (t, (int)re->x1, (int)re->y2, (int)re->x2, (int)re->y2);
}
/**
 * \brief Gets the rectangle's edges
 * \param re a rectangle
 * \param l a SCE_SLine where store the left edge
 * \param r a SCE_SLine where store the right edge
 * \param b a SCE_SLine where store the bottom edge
 * \param t a SCE_SLine where store the top edge
 * 
 * \see SCE_Rectangle_MakeLines()
 */
void SCE_Rectangle_MakeLinesf (SCE_SFloatRect *re, SCE_SLine *l, SCE_SLine *r,
                               SCE_SLine *b, SCE_SLine *t)
{
    SCE_Line_Set (l, re->x1, re->y1, re->x1, re->y2);
    SCE_Line_Set (r, re->x2, re->y1, re->x2, re->y2);
    SCE_Line_Set (b, re->x1, re->y1, re->x2, re->y1);
    SCE_Line_Set (t, re->x1, re->y2, re->x2, re->y2);
}
/**
 * \brief Gets the rectangle's edges
 * \param r a rectangle
 * \param lines an array of 4 SCE_SLine where store the edges.
 * 
 * This function gets the rectangle's edges and stores them into a given array
 * of SCE_SLine. The lines are stored in the following order: left, right,
 * bottom and top.
 * 
 * \see SCE_Rectangle_MakeLines()
 */
void SCE_Rectangle_MakeLinesv (SCE_SIntRect *r, SCE_SLine *lines)
{
    SCE_Rectangle_MakeLines (r, &lines[0], &lines[1], &lines[2], &lines[3]);
}
/**
 * \brief Gets the rectangle's edges
 * \param r a rectangle
 * \param lines an array of 4 SCE_SLine where store the edges.
 * 
 * This function gets the rectangle's edges and stores them into a given array
 * of SCE_SLine. The lines are stored in the following order: left, right,
 * bottom and top.
 * 
 * \see SCE_Rectangle_MakeLinesf()
 */
void SCE_Rectangle_MakeLinesfv (SCE_SFloatRect *r, SCE_SLine *lines)
{
    SCE_Rectangle_MakeLinesf (r, &lines[0], &lines[1], &lines[2], &lines[3]);
}

/* ajoute le 26/02/2008 */
/**
 * \brief Checks if two recangles are crossed
 * \param a a rectangle
 * \param b another rectangle
 * \return SCE_RECT_IN if \p b is inside \p a, SCE_RECT_INTERSECTS if \p a and
 *         \p b are crossed and SCE_RECT_OUT if the two rectangles has no
 *          intersections.
 */
int SCE_Rectangle_Intersects (SCE_SIntRect *a, SCE_SIntRect *b)
{
    int start[2];
    int end[2];
    SCE_SIntRect r;
    
    start[0] = max (a->x1, b->x1);
    start[1] = max (a->y1, b->y1);
    end[0] = min (a->x2, b->x2);
    end[1] = min (a->y2, b->y2);
    r.x1 = start[0];
    r.y1 = start[1];
    r.x2 = end[0];
    r.y2 = end[1];
    
    if ((start[0] > end[0]) || (start[1] > end[1]))
        return SCE_RECT_OUT;
    else if (SCE_Rectangle_Equal (&r, a) || SCE_Rectangle_Equal (&r, b))
        return SCE_RECT_IN;
    else
        return SCE_RECT_INTERSECTS;
}
/**
 * \brief Checks if two recangles are crossed
 * \param a a rectangle
 * \param b another rectangle
 * \return SCE_RECT_IN if \p b is inside \p a, SCE_RECT_INTERSECTS if \p a and
 *         \p b are crossed and SCE_RECT_OUT if the two rectangles has no
 *          intersection.
 * 
 * \see SCE_Rectangle_Intersects()
 */
int SCE_Rectangle_Intersectsf (SCE_SFloatRect *a, SCE_SFloatRect *b)
{
    SCE_TVector2 start;
    SCE_TVector2 end;
    SCE_SFloatRect r;
    
    start[0] = max (a->x1, b->x1);
    start[1] = max (a->y1, b->y1);
    end[0] = min (a->x2, b->x2);
    end[1] = min (a->y2, b->y2);
    r.x1 = start[0];
    r.y1 = start[1];
    r.x2 = end[0];
    r.y2 = end[1];
    
    if ((start[0] > end[0]) || (start[1] > end[1]))
        return SCE_RECT_OUT;
    else if (SCE_Rectangle_Equalf (&r, a) || SCE_Rectangle_Equalf (&r, b))
        return SCE_RECT_IN;
    else
        return SCE_RECT_INTERSECTS;
}

/** @} */
